#include "ns32k/tek6000.h"

#undef CPP_PREDEFINES
#define CPP_PREDEFINES \
 "-Dns32000 -Dns32k -Dns16000 -Dns32016 -DUTek -DUTEK -Dbsd -DBSD -Dmerlin \
  -Asystem(unix) -Asystem(bsd) -Acpu(ns32k) -Amachine(ns32k)"

